package handler_utils

import (
	"context"

	"github.com/aws/aws-lambda-go/lambdacontext"
)

func RequestIDFromLambdaContext(ctx context.Context) *string {
	// Get request ID from context
	if ctx != nil {
		lambdaContext, _ := lambdacontext.FromContext(ctx)
		if lambdaContext != nil {
			return &lambdaContext.AwsRequestID
		}
	}
	return nil
}

func RequestIDFromHeaders(headers map[string]string, requestIDHeaderKey string) *string {
	// Get request ID from parent micro service
	if requestID, ok := headers[requestIDHeaderKey]; ok {
		return &requestID
	}
	return nil
}

func AddRequestIDToHeaders(requestID *string, headers map[string]string, requestIDHeaderKey string) {
	if requestID != nil && headers != nil {
		headers[requestIDHeaderKey] = *requestID
	}
}
